import React from "react";
import SegmentView from "./SegmentView";
import Helper from "../helper.js";

class ChangePassPage extends React.Component {
    constructor() {
        super();

        this.state = {
            oldPass: '',
            newPass: '',
        }
    }

    async onFieldChange(name, e) {
        let value = e.target.value;
        let obj = {};
        obj[name] = value;
        this.setState(obj);
    }

    async onCancelClick() {
        this.props.commonObj.setPage('profile');
    }

    async onFormSubmit(e) {
        e.preventDefault();
        
        let obj = {
            password: this.state.newPass,
            oldPass: this.state.oldPass
        }
        let res = await Helper.fetchAPI('profile', 'PATCH', obj);

        this.setState({
            oldPass: '',
            newPass: ''
        });

        if (res.ok) {
            this.props.commonObj.showPopup('Password updated successfully');
            this.props.commonObj.setPage('profile');
        }
        else {
            this.props.commonObj.showPopup('Could not update password: ' + res.json.message);
        }
    }

    render() {
        return (
            <>
                <SegmentView banner="Change password">
                    <form onSubmit={this.onFormSubmit.bind(this)}>
                        <div className="property">
                            Old password<br />
                            <input type="password" value={this.state.oldPass} onChange={this.onFieldChange.bind(this, 'oldPass')} required></input>
                        </div>
                        <div className="property">
                            New password<br />
                            <input type="password" value={this.state.newPass} onChange={this.onFieldChange.bind(this, 'newPass')} required></input>
                        </div>
                        <button onClick={this.onCancelClick.bind(this)}>Cancel</button>
                        <input type="submit" name="pass" value="Change password"></input>
                    </form>
                </SegmentView>
            </>
        );
    }
}

export default ChangePassPage;