import React from "react";
import Helper from "../helper.js";

class FilterParam extends React.Component {
    constructor() {
        super();

        this.state = {
            value: '',
        }
    }

    componentDidMount() {
        this.setState({
            value: this.props.data.default
        });
    }

    getElement(data) {
        switch(data.inputtype) {
            case 'number':
                return (<input 
                    type="number" 
                    id={'param'+data.name}
                    value={this.state.value}
                    min={data.min || null}
                    max={data.max || null}
                    onChange={this.onValueChange.bind(this)}
                ></input>);
            case 'select':
                return (
                <select 
                    id={'param'+data.name} 
                    value={this.state.value} 
                    onChange={this.onValueChange.bind(this)}
                >
                    {data.options.map((option, i) =>
                        <option key={'paramoption'+i}>{option}</option>
                    )}
                </select>
                );
            default:
                return null;

        }
    }

    correctValueType(val) {
        switch(this.props.data.type) {
            case 'number':
                val = parseFloat(val);
                if(isNaN(val)) val = this.props.data.default;
                if(val < this.props.data.min) val = this.props.data.min;
                if(val > this.props.data.max) val = this.props.data.max;
                break;
            case 'string':
                val = val.toString();
                break;
            default:
                break;
        }

        return val;
    }

    onValueChange(e) {
        let val = this.correctValueType(e.target.value);

        this.setState({
            value: val
        });
        this.props.onChange(this.props.index, val);
    }

    render() {
        let data = this.props.data;

        return (
            <div className="filterParam">
                <label htmlFor={'param'+data.name}>{data.name}: </label>
                {this.getElement(data)}
            </div>
        )
    }
}

export default FilterParam;