import React from "react";
import Helper from "../helper.js";
import FilterParam from "./FilterParam.js";

class FiltersView extends React.Component {
    constructor() {
        super();

        this.state = {
            filters: {},
            selectedFilter: {},
            selectedFilterName: '',
            paramValues: [],
            versions: []
        };
    }

    componentDidMount() {
        this.fetchFilters();
        this.fetchPhotoVersions();
    }

    async fetchPhotoVersions(refetch = false) {
        let photo = this.props.commonObj.selectedPhoto;
        if(!photo) return;

        if(refetch) {
            let res = await Helper.fetchAPI(`photos/${photo.id}`, 'GET');
            if(!res.ok) return;
            this.props.commonObj.selectedPhoto = res.json;
            photo = res.json;
        }

        let versions = photo.history.filter(entry => entry.status === 'filter');
        versions.reverse();
        this.setState({
            versions
        });
    }

    async fetchFilters() {
        let res = await Helper.fetchAPI(`photos/filters`, 'GET');
        if(!res.ok) return;

        let selected = Object.values(res.json)[0];
        this.setState({
            filters: res.json,
            selectedFilter: selected,
            selectedFilterName: Object.keys(res.json)[0],
            paramValues: selected.params.map(param => param.default)
        });
    }

    async onAddFilterChange(e) {
        let filterName = e.target.value;
        let selected = this.state.filters[filterName];

        this.setState({
            selectedFilter: selected,
            selectedFilterName: filterName,
            paramValues: selected.params.map(param => param.default)
        });
    }

    onParamChange(index, value) {
        let paramValues = this.state.paramValues;
        paramValues[index] = value;
        this.setState({
            paramValues
        });
    }

    async onApply(e) {
        let customPathMaybe = this.props.commonObj.getCustomViewPath();
        const obj = {
            photoid: this.props.commonObj.selectedPhoto.id,
            filter: this.state.selectedFilterName,
            params: this.state.paramValues,
            customSource: customPathMaybe || undefined
        };

        let res = await Helper.fetchAPI(`photos/filters`, 'POST', obj);
        if(!res.ok) {
            this.props.commonObj.showPopup('Could not add filter: ' + res.json.message);
        }

        await this.props.commonObj.fetchPhotos();
        this.fetchPhotoVersions(true);
    }

    onVersionClick(version) {
        this.props.commonObj.setCustomViewPath(version.path);
    }

    render() {
        let filterNames = Object.keys(this.state.filters);
        let filterParams = this.state.selectedFilter?.params || [];

        return (
            <>
                <div className="horizontalScroll">
                    {this.state.versions.map((entry, i) => {
                        if(!entry.path) return;
                        return (<div className="image filterImage" key={'version'+i}>
                            <img 
                                src={Helper.filesUrl + entry.path} 
                                alt="version"
                                onClick={this.onVersionClick.bind(this, entry)}
                            ></img>
                        </div>);
                    })}
                </div>
                <div className="horizontalCont">
                    <select className="filterParam" onChange={this.onAddFilterChange.bind(this)}>
                        {filterNames.map((filter, i) => 
                            <option key={'filter'+i}>{filter}</option>
                        )}
                    </select>
                    {
                        filterParams.map((param, i) => 
                            <FilterParam 
                                key={`param${this.state.selectedFilterName}${i}`} 
                                data={param} 
                                onChange={this.onParamChange.bind(this)}
                                index={i}
                            ></FilterParam>
                        )
                    }
                    <button className="filterParam" onClick={this.onApply.bind(this)}>Apply</button>
                </div>
            </>
        );
    }
}

export default FiltersView;