import React from "react";
import SegmentView from "./SegmentView";
import Helper from "../helper.js";
import TagsView from "./TagsView";
import FiltersView from "./FiltersView";

class PhotoPage extends React.Component {
    constructor() {
        super();

        this.state = {
            metadata: [],
            customPath: null,
        };
    }

    componentDidMount() {
        this.fetchMetadata();
    }

    async fetchMetadata() {
        let photo = this.props.commonObj.selectedPhoto;
        let res = await Helper.fetchAPI(`photos/meta/${photo.id}`, 'GET');
        if(!res.ok) return;

        let filtered = Object.entries(res.json).filter(entry =>
            typeof entry[1] !== 'object'
        );

        let timestamp = photo.history[0].timestamp;
        let prettyTime = new Date(timestamp).toLocaleString();
        filtered.push(
            ['id', photo.id, true], 
            ['userID', photo.userID, true],
            ['time uploaded', prettyTime, true]    
        );

        this.setState({
            metadata: filtered
        });
    }

    onCloseClick() {
        this.props.commonObj.setPage('profile');
    }

    async onDeletePhotoClick() {
        let photoID = this.props.commonObj.selectedPhoto.id;
        let res = await Helper.fetchAPI(`photos/${photoID}`, 'DELETE');
        if(!res.ok) {
            this.props.commonObj.showPopup('Error: ' + res.json.message);
        } else {
            this.props.commonObj.setPage('profile');
        }
    }

    setCustomViewPath(path) {
        this.setState({customPath: path});
    }

    getCustomViewPath() {
        return this.state.customPath;
    }

    render() {
        this.props.commonObj.setCustomViewPath = this.setCustomViewPath.bind(this);
        this.props.commonObj.getCustomViewPath = this.getCustomViewPath.bind(this);

        let src = this.state.customPath || this.props.commonObj.selectedPhoto.file.path;
        src = Helper.filesUrl + src;

        return (
            <>
                <SegmentView banner="Photo" wide>
                    <div className="bigImageCont">
                        <button className="closeButton" onClick={this.onCloseClick.bind(this)}></button>
                        <img src={src} alt="photo" className="bigImage"></img>
                        <button className="delPhotoBtn" onClick={this.onDeletePhotoClick.bind(this)}>DELETE PHOTO</button>
                        {
                            this.state.customPath ? 
                                <button onClick={this.setCustomViewPath.bind(this, null)}>Show original</button>
                                : null
                        }
                    </div>
                </SegmentView>
                <SegmentView banner="Filters" wide>
                    <FiltersView commonObj={this.props.commonObj}></FiltersView>
                </SegmentView>
                <SegmentView banner="Tags" wide>
                    <TagsView commonObj={this.props.commonObj}></TagsView>
                </SegmentView>
                <SegmentView banner="Metadata" wide>
                    <div className="horizontalCont">
                    {
                        this.state.metadata.map((entry, index) =>
                            <div className={'property' + (entry[2] ? ' special' : '')} key={'metadata' + index}>
                                {entry[0]}<br />
                                <input type="text" value={entry[1]} disabled></input>
                            </div>
                        )
                    }
                    </div>
                </SegmentView>
            </>
        );
    }
}

export default PhotoPage;