import React from "react";
import Helper from "../helper.js";

class ProfileView extends React.Component {

    constructor() {
        super();

        this.state = {
            editmode: false,
            name: '',
            lastname: '',
            email: '',
            id: '',
            avatar: '',
            loaded: false,
        }
    }

    componentDidMount() {
        this.getProfileInfo();
    }

    async onChangeAvatarClick() {
        const file = await Helper.openImage();
        const formData = new FormData();
        formData.append('file', file);
        await Helper.fetchAPI('profile/avatar', 'POST', null, formData);
        this.getProfileInfo();
    }

    async getProfileInfo() {
        let res = await Helper.fetchAPI('profile', 'GET')
        if (!res.ok) return;

        this.setState({
            name: res.json.name,
            lastname: res.json.lastname,
            email: res.json.email,
            id: res.json.id,
            avatar: res.json.avatar?.path,
            loaded: true
        });
    }

    onInfoSubmit(e) {
        e.preventDefault();
        let type = e.nativeEvent.submitter.name;
        switch(type) {
            case 'info':
                this.onChangeInfoClick();
                break;
            case 'pass':
                this.props.commonObj.setPage('changePass');
                break;
        }
    }

    async onChangeInfoClick() {
        
        this.setState({
            editmode: !this.state.editmode
        });

        if (this.state.editmode) {
            let obj = {
                name: this.state.name,
                lastname: this.state.lastname,
                email: this.state.email
            }
            let res = await Helper.fetchAPI('profile', 'PATCH', obj);
            if (!res.ok)
                this.props.commonObj.showPopup('Could not update profile: ' + res.json.message);
            this.getProfileInfo();
        }
    }

    async onFieldChange(name, e) {
        let value = e.target.value;
        let obj = {};
        obj[name] = value;
        this.setState(obj);
    }

    render() {
        let avatarSrc = this.state.avatar || 'static/avatar.png';
        let editmodeButtonText = this.state.editmode ? 'Apply' : 'Change profile info';

        return (
            !this.state.loaded ? <div>Loading...</div> :
                <div className="horizontalCont">
                    <div className="profileInfo">
                        <form onSubmit={this.onInfoSubmit.bind(this)}>
                            <div className="property">
                                Name<br />
                                <input type="text" value={this.state.name} disabled={!this.state.editmode} onChange={this.onFieldChange.bind(this, 'name')} required></input>
                            </div>
                            <div className="property">
                                Lastname<br />
                                <input type="text" value={this.state.lastname} disabled={!this.state.editmode} onChange={this.onFieldChange.bind(this, 'lastname')} required></input>
                            </div>
                            <div className="property">
                                E-mail<br />
                                <input type="email" value={this.state.email} disabled={!this.state.editmode} onChange={this.onFieldChange.bind(this, 'email')} required></input>
                            </div>
                            <div className="property">
                                ID<br />
                                <input type="text" value={this.state.id} disabled={true}></input>
                            </div>
                            <input type="submit" name="info" value={editmodeButtonText}></input>
                            <input type="submit" name="pass" value="Change password"></input>
                        </form>
                    </div>
                    <div className="profileAvatar">
                        <img src={Helper.filesUrl + avatarSrc} alt="avatar"></img><br />
                        <button onClick={this.onChangeAvatarClick.bind(this)}>Change avatar</button>
                    </div>
                </div >
        );
    }

}

export default ProfileView;