import React from "react";
import Helper from "../helper.js";

class TagsView extends React.Component {
    constructor() {
        super();

        this.state = {
            newTag: '',
            tags: [],
            tagOpts: [],
            displayOpts: false
        };
    }

    componentDidMount() {
        this.fetchTags();
        this.fetchTagOpts();
    }

    async fetchTags() {
        let photoID = this.props.commonObj.selectedPhoto.id;
        let res = await Helper.fetchAPI(`photos/tags/${photoID}`, 'GET');
        if(!res.ok) return;
        this.setState({
            tags: res.json
        });
    }

    async fetchTagOpts() {
        let res = await Helper.fetchAPI(`tags`, 'GET');
        if(!res.ok) return;
        res.json.sort((a, b) => b.popularity - a.popularity);
        this.setState({
            tagOpts: res.json
        });
    }

    async onDeleteTagClick(tagName) {
        let obj = {
            photoid: this.props.commonObj.selectedPhoto.id,
            tag: tagName
        };

        let res = await Helper.fetchAPI(`photos/tags`, 'DELETE', obj);
        if(!res.ok) {
            this.props.commonObj.showPopup('Error: ' + res.json.message);
        }

        this.fetchTags();
    }

    async onAddTagClick() {
        let obj = {
            photoid: this.props.commonObj.selectedPhoto.id,
            tag: this.state.newTag
        };

        let res = await Helper.fetchAPI(`photos/tags`, 'PATCH', obj);
        if(!res.ok) {
            this.props.commonObj.showPopup('Error: ' + res.json.message);
        }

        this.fetchTags();
        this.fetchTagOpts();

        this.setState({
            newTag: ''
        });
    }

    onNewTagChange(e) {
        this.setState({
            newTag: e.target.value
        });
    }

    onNewTagKeyDown(e) {
        if(e.key === 'Enter') {
            this.onAddTagClick();
        }
    }

    onNewTagFocus(focused, e) {
        if(focused)
            this.fetchTagOpts();
        
        setTimeout(() => {
            this.setState({
                displayOpts: focused
            });
        }, 200);
    }

    onOptClick(tagName) {
        this.setState({
            newTag: tagName
        }, () => {
            this.onAddTagClick();
        });
    }

    render() {
        let optsStyle = {
            display: this.state.displayOpts ? 'block' : 'none'
        }

        return (
            <div className="horizontalCont">
                <div className="tag">
                    <input 
                        type="text" 
                        placeholder="Add tag" 
                        value={this.state.newTag} 
                        onChange={this.onNewTagChange.bind(this)}
                        onKeyDown={this.onNewTagKeyDown.bind(this)}
                        onFocus={this.onNewTagFocus.bind(this, true)}
                        onBlur={this.onNewTagFocus.bind(this, false)}
                    ></input>
                    <div className="tagOptsList" style={optsStyle}>
                        {
                            this.state.tagOpts.map((tag, index) =>
                                <div key={'tagopt'+index} onClick={this.onOptClick.bind(this, tag.name)}>
                                    <div>
                                        {tag.name}
                                    </div>
                                    <div className="tagPopularity">
                                        {tag.popularity}
                                    </div>
                                </div>
                            )
                        }
                    </div>
                    <button className="addTagBtn" onClick={this.onAddTagClick.bind(this)}></button>
                </div>
                {
                    this.state.tags.map((tag, index) =>
                        <div className="tag" key={'tag' + index}>
                            <div className="tagText">{tag.name}</div>
                            <button className="delTagBtn" onClick={this.onDeleteTagClick.bind(this, tag.name)}></button>
                        </div>
                    )
                }
            </div>
        );
    }
}

export default TagsView;