const photos = {};
const tags = {};
let currentPhotoID = 0;

function getPhotos(userID) {
    return Object.values(photos).filter(photo => photo.userID == userID);
}

function getPhoto(id) {
    return photos[id];
}

function getPhotoTags(id) {
    let photo = photos[id];
    if (!photo) return false;
    if (!photo.tags) return [];
    return photo.tags.map(tagID => getTag(tagID));
}

function addPhoto(data, userID) {
    const obj = {
        file: data.file,
        id: currentPhotoID,
        userID: userID,
        history: [
            {
                status: 'original',
                timestamp: Date.now()
            }
        ]
    }
    photos[currentPhotoID] = obj;
    currentPhotoID++;
    return obj
}

function delPhoto(id) {
    let photo = photos[id];
    if(!photo) return false;
    let path = photo.file.path;
    if(photo.tags) photo.tags.forEach(tag => {
        tags[tag]--;
        if(tags[tag] == 0) delete tags[tag];
    });
    delete photos[id];
    return path;
}

function modifyPhotoJSON(id, data) {
    let obj = photos[id]
    if (!obj) return false;
    Object.assign(obj, data);
    obj.history.push({
        status: 'modified',
        timestamp: Date.now()
    })
    return true;
}

function addFilterHistory(id, name, path) {
    let photo = photos[id]
    if (!photo) return false;
    photo.history.push({
        status: 'filter',
        name: name,
        path: path,
        timestamp: Date.now()
    })
    return true;
}

function getTags(raw) {
    if (raw) return Object.keys(tags);
    return Object.entries(tags).map(entry => ({
        name: entry[0],
        popularity: entry[1]
    }));
}

function getTag(name) {
    return {
        name: name,
        popularity: tags[name]
    }
}

function addTag(name) {
    if(tags[name]) return false;

    tags[name] = 0;
    return true;
}

function addTagsToPhoto(photoID, tagsArray) {
    let photo = photos[photoID];
    if (!photo) return false;
    if (!photo.tags) photo.tags = [];
    tagsArray.forEach(tag => {
        if (photo.tags.includes(tag)) return;
        photo.tags.push(tag);

        if(!tags[tag]) tags[tag] = 0;
        tags[tag]++;
    });
    return true;
}

function removeTagsFromPhoto(photoID, tagsArray) {
    let photo = photos[photoID];
    if (!photo) return false;
    if (!photo.tags) return true;
    tagsArray.forEach(tag => {
        if (!photo.tags.includes(tag)) return;
        photo.tags.splice(photo.tags.indexOf(tag), 1);
        
        tags[tag]--;
        if(tags[tag] == 0) delete tags[tag];
    });
    return true;
}


module.exports = { getPhotos, getPhoto, getPhotoTags, addPhoto, delPhoto, modifyPhotoJSON, addFilterHistory, getTags, getTag, addTag, addTagsToPhoto, removeTagsFromPhoto }