const formidable = require('formidable');
const sharp = require('sharp');
const fs = require('fs');
const UPLOAD_DIR = './upload/';

function sendPhoto(req) {
    return new Promise((resolve, reject) => {
        const form = formidable({ keepExtensions: true });
        form.uploadDir = UPLOAD_DIR;
        form.parse(req, function (err, fields, file) {
            if (err) {
                console.error(err);
            }
            resolve({ file, fields, err });
        });
    });
}

//unused
function renameFile(path, newPath) {
    return new Promise((resolve, reject) => {
        fs.rename(UPLOAD_DIR + path, UPLOAD_DIR + newPath, (err) => {
            if (err) {
                console.error(err);
                resolve(false);
            }
            resolve(true);
        })
    });
}

function deletePhoto(filepath) {
    return new Promise((resolve, reject) => {
        fs.unlink(filepath, (err) => {
            if (err) {
                console.error(err)
                resolve(false);
            }
            resolve(true);
        })
    })
}

async function getMeta(path) {
    let meta = await sharp(path).metadata();
    return meta;
}

function getReadStream(path) {
    let filePath = './' + path;
    if(!fs.existsSync(filePath)) return null;
    let stat = fs.statSync(filePath);
    let readStream = fs.createReadStream(filePath);
    return [stat, readStream];
}

const filters = require('./filters.json');

function getAvailableFilters() {
    return filters;
}

async function applyFilter(path, filterName, filterParams) {
    let filter = filters[filterName];
    if(!filter) return -1;
    if(filter.params && !filterParams) return -2;
    if(filterParams.length != filter.params.length) return -2;
    if(!fs.existsSync(path)) return -3;

    filter.params.forEach((param, i) => {
        if(param.type != typeof filterParams[i]) return -4;
        if(param.options && !param.options.includes(filterParams[i])) return -5;
    });
    
    let outpath;
    let outnumber = 0;
    let ext = path.split('.').pop();
    let noExt = path.split('.').slice(0, -1).join('.');
    if(filterName == 'toFormat') ext = filterParams[0].toLowerCase();
    do {
        outpath = noExt + '_' + filterName + '_' + outnumber + '.' + ext;
        outnumber++;
    }
    while(fs.existsSync(outpath));

    if(filter.asObject) {
        let obj = {};
        filterParams.forEach((param, i) => {
            obj[filter.params[i].name] = param;
        });
        filterParams = [obj];
    } 
    else if(filter.fixedparams)
        filterParams.push(...filter.fixedparams);

    try {
        await sharp(path)
        [filterName](...filterParams)
        .toFile(outpath);

    } catch (err) {
        return -6;
    }
   
    return outpath;
}

module.exports = { sendPhoto, deletePhoto, getMeta, getReadStream, getAvailableFilters, applyFilter }