const usersController = require('./usersController');
const fileController = require('./fileController');

class HTTPRouter {
    constructor() {
        this.lastReq;
        this.routes = [];
    }

    addRoutes(routesArray) {
        this.routes.push(...routesArray);
    }

    handleFile(req, res) {
        if (req.method !== 'GET') return false;
        if (!req.url.startsWith('/file/')) return false;
        let path = req.url.slice(5);
        
        let fcrs = fileController.getReadStream(path);
        if(!fcrs) return false;
        const [stat, readStream] = fcrs;

        res.writeHead(200, {
            'Content-Length': stat.size
        });

        readStream.pipe(res);
        return true;
    }

    handleOptions(req, res) {
        if (req.method !== 'OPTIONS') return false;

        res.writeHead(200);
        res.end();
        return true;
    }

    async handleReq(req, res) {
        this.lastReq = req;

        res.setHeader('Access-Control-Allow-Origin', '*');
        res.setHeader('Access-Control-Allow-Headers', '*');
        res.setHeader('Access-Control-Allow-Methods', 'OPTIONS, GET, POST, PATCH, PUT, DELETE');
        if(this.handleFile(req, res)) return;
        if(this.handleOptions(req, res)) return;
        res.setHeader('Content-Type', 'application/json');

        let retValue = [404, { error: 'not found' }];
        try {
            for (let i = 0; i < this.routes.length; i++) {
                if (req.method != this.routes[i].method) continue;
                if (!this.routes[i].fun) continue;
                if (!req.url.startsWith(this.routes[i].url)) continue;
                if (this.routes[i].loginRequired && !usersController.checkLogin(req)) {
                    retValue = [403, { message: 'login required', url: this.routes[i].url }];
                    break;
                }
                if (this.routes[i].paramAfterURL) {
                    let param = req.url.slice(this.routes[i].url.length);
                    retValue = await this.routes[i].fun(req, param);
                    break;
                }
                if (req.url != this.routes[i].url) continue;
                retValue = await this.routes[i].fun(req);
                break;
            }
        } catch (error) {
            retValue = [500, { message: "internal server error" }];
        }

        console.log(retValue);
        res.statusCode = retValue[0];
        let json = JSON.stringify(retValue[1], null, 5);
        res.end(json);
    }
}

module.exports = HTTPRouter