const usersController = require('./usersController');
const fileController = require('./fileController');
const postParser = require('./postParser');

const routes = [
    {
        method: 'GET',
        url: '/api/profile',
        loginRequired: true,
        fun: async (req) => {
            let userID = usersController.getCurrentUserID(req);
            let user = usersController.getUser(userID);
            return [200, user];
        },
    },
    {
        method: 'PATCH',
        url: '/api/profile',
        loginRequired: true,
        fun: async (req) => {
            let obj = await postParser.parseBody(req, true);
            if (!obj || Object.keys(obj).length == 0)
                return [400, { message: 'unable to parse JSON' }];

            let userID = usersController.getCurrentUserID(req);
            let code = await usersController.updateUser(userID, obj);

            switch (code) {
                case 0:
                    return [200, { message: 'success' }];
                case 1:
                    return [404, { message: 'user not found' }];
                case 2:
                    return [400, { message: 'user with this e-mail already exists' }];
                case 3:
                    return [400, { message: 'wrong password' }];
            }
        }
    },
    {
        method: 'POST',
        url: '/api/profile/avatar',
        loginRequired: true,
        fun: async (req) => {
            let userID = usersController.getCurrentUserID(req);
            let saveStatus = await fileController.sendPhoto(req);
            if (saveStatus.err || !saveStatus.file || !saveStatus.file.file)
                return [500, { message: 'error while uploading' }];

            usersController.addAvatar(userID, saveStatus.file.file);
            return [200, { message: 'success' }];
        },
    }
];

module.exports = routes;