const usersController = require('./usersController');
const postParser = require('./postParser');

const routes = [
    {
        method: 'POST',
        url: '/api/user/register',
        fun: async (req) => {
            let obj = await postParser.parseBody(req, true);
            if (!obj || Object.keys(obj).length == 0)
                return [400, { message: 'unable to parse JSON' }];
            if (!obj.name || !obj.lastname || !obj.email || !obj.password) {
                return [400, { message: 'required fields: name, lastname, email, password' }];
            }
            let userobj = await usersController.registerUser(obj.name, obj.lastname, obj.email, obj.password);
            if (!userobj) {
                return [409, { message: 'user with this e-mail already exists' }];
            }

            let token = usersController.createToken(userobj.email, 'confirm');

            return [200, { token }];
        },
    },
    {
        method: 'GET',
        url: '/api/user/confirm/',
        paramAfterURL: true,
        fun: async (req, param) => {
            if (!param) return [400, { message: 'no token provided' }];
            let success = usersController.confirmUser(param);
            return success ? [200, { message: 'user confirmed' }] : [400, { message: 'bad or expired token' }];
        },
    },
    {
        method: 'POST',
        url: '/api/user/login',
        fun: async (req) => {
            let obj = await postParser.parseBody(req, true);
            if (!obj || Object.keys(obj).length == 0)
                return [400, { message: 'unable to parse JSON' }];
            if (!obj.email || !obj.password) {
                return [400, { message: 'required fields: email, password' }];
            }
            let session = await usersController.loginUser(obj.email, obj.password);
            switch (session) {
                case 0:
                    return [400, { message: 'no user with this e-mail exists' }];
                    break;
                case 1:
                    return [400, { message: 'wrong password' }];
                    break;
                case 2:
                    return [403, { message: 'user is not confirmed' }];
                    break;
            }
            return [200, { message: 'logged in', token: session }];
        }
    },
    {
        method: 'GET',
        url: '/api/user/check',
        loginRequired: true,
        fun: async (req) => [200, { message: 'success' }]
    },
    {
        method: 'GET',
        url: '/api/user',
        fun: async (req) => {
            let users = usersController.debugGetUsers();
            return [200, users];
        }
    },
    {
        method: 'GET',
        url: '/api/user/logout',
        loginRequired: true,
        fun: async (req) => {
            let users = usersController.logout(req);
            return [200, { message: 'success' }];
        }
    }
]

module.exports = routes;